Object.defineProperty(exports, '__esModule', { value: true });

var sugarHigh = require('sugar-high');
var unistUtilMap = require('unist-util-map');
var rangeParser = require('parse-numeric-range');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var rangeParser__default = /*#__PURE__*/_interopDefault(rangeParser);

const keywords$1 = new Set([
    'and',
    'as',
    'assert',
    'async',
    'await',
    'break',
    'class',
    'continue',
    'def',
    'del',
    'elif',
    'else',
    'except',
    'finally',
    'for',
    'from',
    'global',
    'if',
    'import',
    'in',
    'is',
    'lambda',
    'nonlocal',
    'not',
    'or',
    'pass',
    'raise',
    'return',
    'try',
    'while',
    'with',
    'yield'
]);
const onCommentStart$1 = (currentChar, _nextChar)=>{
    return currentChar === '#';
};
const onCommentEnd$1 = (_prevChar, currChar)=>{
    return currChar === '\n';
};

var python = {
  __proto__: null,
  keywords: keywords$1,
  onCommentEnd: onCommentEnd$1,
  onCommentStart: onCommentStart$1
};

const keywords = new Set([
    // css keywords like @media, @import, @keyframes, etc.
    '@media',
    '@import',
    '@keyframes',
    '@font-face',
    '@supports',
    '@page',
    '@counter-style',
    '@font-feature-values',
    '@viewport',
    '@counter-style',
    '@font-feature-values',
    '@document'
]);
const onCommentStart = (currentChar, nextChar)=>{
    return '/*' === currentChar + nextChar;
};
const onCommentEnd = (prevChar, currChar)=>{
    return '*/' === prevChar + currChar;
};

var css = {
  __proto__: null,
  keywords: keywords,
  onCommentEnd: onCommentEnd,
  onCommentStart: onCommentStart
};

var languagesOptions = {
  __proto__: null,
  css: css,
  python: python
};

function cx(...args) {
    return args.filter(Boolean).join(' ');
}
/**
 * Parses highlight information from the `meta` field of a Remark code block node.
 * @param meta The `meta` string containing highlight information, e.g., "{1,3-5}".
 * @returns An array of highlight ranges. Single numbers represent single lines, and tuples represent ranges.
 */ function parseHighlightMeta(meta) {
    if (!meta) return [];
    const highlightRegex = /{([\d,-]+)}/ // Matches {1,3-5}
    ;
    const match = meta.match(highlightRegex);
    if (!match) return [];
    const ranges = match[1].split(',').map((range)=>{
        if (range.includes('-')) {
            const [start, end] = range.split('-').map(Number);
            return [
                start,
                end
            ];
        }
        return Number(range);
    });
    return ranges;
}
const parseLang = (str)=>{
    const match = (regexp)=>{
        const m = (str || '').match(regexp);
        return Array.isArray(m) ? m.filter(Boolean) : [];
    };
    const [lang = 'unknown'] = match(/^[a-zA-Z\d-]*/g);
    const range = rangeParser__default.default(match(/\{(.*?)\}$/g).join(',').replace(/^\{/, '').replace(/\}$/, ''));
    return {
        lang,
        range
    };
};
const h = (type, attrs, children)=>{
    return {
        type: 'element',
        tagName: type,
        data: {
            hName: type,
            hProperties: attrs,
            hChildren: children
        },
        properties: attrs,
        children
    };
};
const highlight = ()=>(tree)=>{
        return unistUtilMap.map(tree, (node)=>{
            const { type, tagName } = node;
            if (tagName !== 'code' && type !== 'code') return node;
            const { lang } = parseLang(node.lang);
            const highlightRanges = parseHighlightMeta(node.meta);
            let options = undefined;
            if (lang in languagesOptions) {
                options = languagesOptions[lang];
            }
            const codeText = node.value || node.children.filter(({ type })=>type === 'text').map(({ value })=>value).pop();
            const childrenLines = sugarHigh.generate(sugarHigh.tokenize(codeText, options));
            let lineIndex = 1;
            for (const line of childrenLines){
                // if it's highlighted lines, add a classname `sh__line--highlighted`
                let isCurrentLineHighlighted = highlightRanges.includes(lineIndex);
                let highLightClassName = '';
                if (isCurrentLineHighlighted) {
                    highLightClassName = 'sh__line--highlighted';
                }
                // add line break
                line.children.push(h('span', {
                    className: `sh__token--line`
                }, [
                    {
                        type: 'text',
                        value: '\n'
                    }
                ]));
                // add class to line
                line.properties.className = cx(line.properties.className, highLightClassName);
                lineIndex++;
            }
            const code = h('code', {
                className: `sh-lang--${lang}`,
                ['data-sh-language']: `${lang}`
            }, childrenLines);
            const pre = h('pre', {
                className: `sh-lang--${lang}`
            }, [
                code
            ]);
            return pre;
        });
    };

exports.default = highlight;
exports.highlight = highlight;
